#!/usr/bin/env python

# EveryGui (C)2006 Jeffrey Bakker


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import libglade
import gtk

import locale
locale.setlocale(locale.LC_ALL,'')

import shutil
import os
import re


# TODO: combobox selection on new option, program combo

# option info ==================================================================
class optsTab:

	def __init__(self, name, order, type, label, switch, tip, display=0,
			x1='', x2='', x3=''):

		# display order ------------------------------------------------
		self.display = display

		# common -------------------------------------------------------
		self.name = name
		self.order = int(order)
		self.type = type
		self.label = label
		self.switch = switch
		self.tip = tip

		# extras -------------------------------------------------------
		self.ex1 = x1
		self.ex2 = x2
		self.ex3 = x3

# dialog for program info ======================================================
class DlgProg(libglade.GladeWrapper):

	def __init__(self, Filename, WindowName, Pinfo, start_dir="."):
		libglade.GladeWrapper.__init__(self, Filename, WindowName)

		self.entProgName.set_text(Pinfo[0])
		self.entProgForm.set_text(Pinfo[1])
		self.entProgDesc.set_text(Pinfo[2])


	def get_info(self):

		Pn = self.entProgName.get_text()
		Pc = self.entProgForm.get_text()
		Pd = self.entProgDesc.get_text()

		Pinfo = [Pn, Pc, Pd]
		return Pinfo

	def progcancel_clicked(self, widget, data=None):

		self.dlgProgInfo.hide_all()

	def progedit_clicked(self, widget, data=None):

		self.dlgProgInfo.hide_all()


# dialog for adding a new option ===============================================
class DlgAdd(libglade.GladeWrapper):

	def __init__(self, Filename, WindowName, Extra="", start_dir="."):
		libglade.GladeWrapper.__init__(self, Filename, WindowName)

		self.ex = Extra
		self.xmlex = None

		self.xcomn = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblCommon")
		self.common = self.xcomn.get_widget('tblCommon')

		if self.ex == 'Label':

			self.xcomn.get_widget('spnOrder').set_sensitive(False)
			self.xcomn.get_widget('entSwitch').set_sensitive(False)
			self.xcomn.get_widget('entTooltip').set_sensitive(False)

		if self.ex == 'FileChooserButton':

			self.xmlex = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="vbExtrasFile")
			self.extras = self.xmlex.get_widget('vbExtrasFile')

#		elif self.ex == 'RadioButton':

#			self.xmlex = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblExtrasRadio")
#			self.extras = self.xmlex.get_widget('tblExtrasRadio')

		elif self.ex == 'SpinButton' or self.ex == 'HScale':

			self.xmlex = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblExtrasSpin")
			self.extras = self.xmlex.get_widget('tblExtrasSpin')

		else:

			self.extras = gtk.Label("The option type " + self.ex + " has no extra parameters")

		self.extras.show()

		self.vbAddOpt.pack_start(self.common)
		self.vbAddOpt.pack_start(self.extras)

	# return option info ===================================================
	def get_info(self):

		type = self.ex

		# common -------------------------------------------------------
		order = self.xcomn.get_widget('spnOrder').get_value_as_int()
		label = self.xcomn.get_widget('entLabel').get_text()
		switch = self.xcomn.get_widget('entSwitch').get_text()
		tip = self.xcomn.get_widget('entTooltip').get_text()

		label = label.replace(',','')
		switch = switch.replace(',','')
		tip = tip.replace(',','')

		name = type + "_" + switch + " " + label


		# extras -------------------------------------------------------
		if self.ex == 'FileChooserButton':

			if self.xmlex.get_widget('rbOpenFile').get_active():

				ex = "OPEN"

			elif self.xmlex.get_widget('rbSelectDir').get_active():

				ex = "SELECT_FOLDER"

			else:
				ex = "OPEN"

			return optsTab(name,order,type,label,switch,tip,0,ex)

#		elif self.ex == 'RadioButton':

#			ex = self.xmlex.get_widget('entRadGroup').get_text()

#			return optsTab(name,order,type,label,switch,tip,0,ex)

		elif self.ex == 'SpinButton' or self.ex == 'HScale':

			ex1 = self.xmlex.get_widget('entDefVal').get_text()
			ex2 = self.xmlex.get_widget('entMinVal').get_text()
			ex3 = self.xmlex.get_widget('entMaxVal').get_text()

			return optsTab(name,order,type,label,switch,tip,0,ex1,ex2,ex3)

		else:
			return optsTab(name,order,type,label,switch,tip,0)


	def addcancel_clicked(self, widget, data=None):

		self.dlgAdd.hide_all()

	def addopt_clicked(self, widget, data=None):

		self.dlgAdd.hide_all()


# designer class from GladeWrapper =============================================
class Designer(libglade.GladeWrapper):


	# grab widgets from xml ================================================
	def __init__(self, Filename, WindowName, start_dir="."):
		libglade.GladeWrapper.__init__(self, Filename, WindowName)


		self.slot = 0

		self.PName = ''
		self.PComm = ''
		self.PDesc = ''

		self.tooltips = gtk.Tooltips()
		self.selected = ''
		self.egdfile = ''

		self.cwd = os.getcwd()
		self.egd_dir = os.environ["HOME"] + "/.egconfig"

		self.applocs = {}
		self.appconf = {}

		self.Widgets = {}
		self.Options = {}

		if not os.path.exists(self.egd_dir):
			shutil.copytree("/usr/local/share/everygui/egconfig", self.egd_dir)

		# read program configs into dictionaries and combo box ---------
		self.cbProgram.remove_text(0)
		self.cbProgram.append_text('Editing Current Definition')

		os.chdir(self.egd_dir)
		egd_dir = os.listdir('.')
		egd_dir.sort()

		for egd in egd_dir:

			try:
				egd_file = open(egd, "r")

			except IOError:
				print "Cannot open file", egd

			app_info = egd_file.readline()
#			print "App info:", app_info

			dump, info = app_info.split('^')
			appname, location, about = info.split(',')

			self.appconf[appname] = egd
			self.applocs[appname] = location
			self.cbProgram.append_text(appname)

			egd_file.close()

		os.chdir(self.cwd)

		self.cbProgram.set_active(0)


		# preview area -------------------------------------------------
		self.vb = gtk.VBox(True, 2)
		self.vb.set_border_width(8)
		self.vb.show()

		self.vpPreview.add(self.vb)

		# edit panel ---------------------------------------------------
		self.xmlex = {}

		self.xcomn = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblCommon")
		self.common = self.xcomn.get_widget('tblCommon')

#		self.xmlex["radb"] = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblExtrasRadio")
		self.xmlex["spin"] = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="tblExtrasSpin")
		self.xmlex["file"] = gtk.glade.XML('/usr/local/share/everygui/ui/egdesigner.glade', root="vbExtrasFile")

		self.xcomn.get_widget('spnOrder').set_sensitive(False)
		self.xcomn.get_widget('entLabel').set_sensitive(False)
		self.xcomn.get_widget('entSwitch').set_sensitive(False)
		self.xcomn.get_widget('entTooltip').set_sensitive(False)

		self.btnDelete.set_sensitive(False)
		self.btnApplyEditOpt.set_sensitive(False)

		self.vbOptInfo.pack_start(self.common)

		if self.selected == '' or self.xmlex == None:
			self.extras = gtk.Label("Nothing selected")

		self.extras.show()
		self.vbOptInfo.pack_start(self.extras)

		self.cbOption.set_active(0)


	# option created =======================================================
	def new_option(self, type):

		add = DlgAdd('/usr/local/share/everygui/ui/egdesigner.glade', "dlgAdd", type)
		add.dlgAdd.hide()

		resp = add.dlgAdd.run()
		if resp == gtk.RESPONSE_OK:

			if self.selected != '':
				self.Widgets[self.selected].drag_unhighlight()

			self.selected = self.build_option(add.get_info())
#			self.Widgets[self.selected].connect("clicked", self.widget_selected)

#			self.cbOption.append_text(self.selected)
			self.new_selection(self.Options[self.selected])


	# change option info and widget from user input ========================
	def update_option(self):

		type = self.Options[self.selected].type

		# get user input -----------------------------------------------
		order = self.xcomn.get_widget('spnOrder').get_value_as_int()
		label = self.xcomn.get_widget('entLabel').get_text()
		switch = self.xcomn.get_widget('entSwitch').get_text()
		tip = self.xcomn.get_widget('entTooltip').get_text()

		label = label.replace(',','')
		switch = switch.replace(',','')
		tip = tip.replace(',','')

		# basic/common updates -----------------------------------------
		self.Options[self.selected].order = order
		self.Options[self.selected].label = label
		self.Options[self.selected].switch = switch
		self.Options[self.selected].tip = tip


		# widget-specific common ---------------------------------------
		if type == 'CheckButton':

			self.Widgets[self.selected].set_label(label)
			self.tooltips.set_tip(self.Widgets[self.selected], tip)

		elif type == 'Label':

			self.Widgets[self.selected].set_text('<b>' + label + '</b>')
			self.Widgets[self.selected].set_use_markup(True)

		# widget-specific extras ---------------------------------------
		elif type == 'FileChooserButton':

			self.Widgets[self.selected + "_lab"].set_label(label)
			self.Widgets[self.selected].set_title(tip)

			if self.xmlex["file"].get_widget('rbOpenFile').get_active():

				self.Widgets[self.selected].set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
				self.Options[self.selected].ex1 = 'OPEN'

			elif self.xmlex["file"].get_widget('rbSelectDir').get_active():

				self.Widgets[self.selected].set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
				self.Options[self.selected].ex1 = 'SELECT_FOLDER'

#		elif type == 'RadioButton':

#			ex1 = self.xmlex["radb"].get_widget('entRadGroup').get_text()

#			self.Widgets[self.selected].set_label(label)
#			self.tooltips.set_tip(self.Widgets[self.selected], tip)

#			self.Widgets[self.selected].set_group(ex1)
#			self.Options[self.selected].ex1 = ex1

		elif type == 'SpinButton' or type == 'HScale':

			ex1 = self.xmlex["spin"].get_widget('entDefVal').get_text()
			ex2 = self.xmlex["spin"].get_widget('entMinVal').get_text()
			ex3 = self.xmlex["spin"].get_widget('entMaxVal').get_text()

			self.Widgets[self.selected + "_lab"].set_label(label)

			if type == 'SpinButton':
				self.tooltips.set_tip(self.Widgets[self.selected], tip)

			self.Widgets[self.selected].set_value(int(ex1))
			self.Widgets[self.selected].set_range(int(ex2), int(ex3))

			self.Options[self.selected].ex1 = ex1
			self.Options[self.selected].ex2 = ex2
			self.Options[self.selected].ex3 = ex3

		else:
			self.Widgets[self.selected + "_lab"].set_label(label)
			self.tooltips.set_tip(self.Widgets[self.selected], tip)


	# no option/widget selected ============================================
	def none_selected(self):

		if self.selected != '' and self.Widgets.has_key(self.selected):
			self.Widgets[self.selected].drag_unhighlight()

		self.selected = ''

		# clear fields -------------------------------------------------
		self.xcomn.get_widget('spnOrder').set_value(1)
		self.xcomn.get_widget('entLabel').set_text('')
		self.xcomn.get_widget('entSwitch').set_text('')
		self.xcomn.get_widget('entTooltip').set_text('')

		# disable editing ----------------------------------------------
		self.xcomn.get_widget('spnOrder').set_sensitive(False)
		self.xcomn.get_widget('entLabel').set_sensitive(False)
		self.xcomn.get_widget('entSwitch').set_sensitive(False)
		self.xcomn.get_widget('entTooltip').set_sensitive(False)

		self.btnDelete.set_sensitive(False)
		self.btnApplyEditOpt.set_sensitive(False)

		self.vbOptInfo.remove(self.extras)
		del self.extras

		self.extras = gtk.Label("Nothing selected")
		self.vbOptInfo.pack_start(self.extras)
		self.extras.show()

		self.cbOption.set_active(0)


	# new widget/option selected ===========================================
	def new_selection(self, optsobj):

		if optsobj.type != 'Label':

			# enable editing ---------------------------------------
			self.xcomn.get_widget('spnOrder').set_sensitive(True)
			self.xcomn.get_widget('entSwitch').set_sensitive(True)
			self.xcomn.get_widget('entTooltip').set_sensitive(True)

		else:
			# label editing ----------------------------------------
			self.xcomn.get_widget('spnOrder').set_sensitive(False)
			self.xcomn.get_widget('entSwitch').set_sensitive(False)
			self.xcomn.get_widget('entTooltip').set_sensitive(False)

		self.xcomn.get_widget('entLabel').set_sensitive(True)

		self.btnDelete.set_sensitive(True)
		self.btnApplyEditOpt.set_sensitive(True)


		# populate fields with info ------------------------------------
		self.xcomn.get_widget('spnOrder').set_value(optsobj.order)
		self.xcomn.get_widget('entLabel').set_text(optsobj.label)
		self.xcomn.get_widget('entSwitch').set_text(optsobj.switch)
		self.xcomn.get_widget('entTooltip').set_text(optsobj.tip)


		# create proper fields for extra editing -----------------------
#		self.extras.destroy()

		self.vbOptInfo.remove(self.extras)
		del self.extras

		if optsobj.type == 'FileChooserButton':

			self.extras = self.xmlex["file"].get_widget('vbExtrasFile')

			if optsobj.ex1 == 'OPEN':
				self.xmlex["file"].get_widget('rbOpenFile').set_active(True)

			elif optsobj.ex1 == 'SELECT_FOLDER':
				self.xmlex["file"].get_widget('rbSelectDir').set_active(True)


#		elif optsobj.type == 'RadioButton':

#			self.extras = self.xmlex["radb"].get_widget('tblExtrasRadio')
#			self.xmlex["radb"].get_widget('entRadGroup').set_text(optsobj.ex1)


		elif optsobj.type == 'SpinButton' or optsobj.type == 'HScale':

			self.extras = self.xmlex["spin"].get_widget('tblExtrasSpin')

			self.xmlex["spin"].get_widget('entDefVal').set_text(optsobj.ex1)
			self.xmlex["spin"].get_widget('entMinVal').set_text(optsobj.ex2)
			self.xmlex["spin"].get_widget('entMaxVal').set_text(optsobj.ex3)

		else:

			self.extras = gtk.Label("The option type " + optsobj.type + " has no extra parameters")

		self.extras.show()
		self.vbOptInfo.pack_start(self.extras)

		self.Widgets[self.selected].drag_highlight()


	# rebuild all widgets ==================================================
	def rebuild_widgets(self):

		for widget in self.Widgets.values():

			widget.destroy()

		self.Widgets.clear()

		for opt in self.Options.values():

			self.build_option(opt, True)


	# reset widgets, options, program info =================================
	def clear_all(self):

		# program info -------------------------------------------------
		self.PName = ''
		self.PComm = ''
		self.PDesc = ''

		# designer info ------------------------------------------------
		self.selected = ''
		self.egdfile = ''
		self.slot = 0

		self.none_selected()

		# widgets & options --------------------------------------------
		if len(self.Widgets) > 0:
			for widget in self.Widgets.values():
				widget.destroy()

		self.Widgets.clear()
		self.Options.clear()

		# selected option combobox -------------------------------------
		self.cbOption.get_model().clear()
		self.cbOption.append_text('Nothing selected')
		self.cbOption.set_active(0)

#		self.cbProgram.set_active(0)

		# TODO: may need to reset tooltips


	# read definition file =================================================
	def open_definition(self, filename):

		try:
			egd = open(filename, "r")

		except IOError:

			print "ERROR: Cannot open file", filename
			return

		self.clear_all()
		s = self.slot

		# program info -------------------------------------------------
		PInfo = egd.readline()
		discard, PInfo = PInfo.split('^')

		self.PName, self.PComm, self.PDesc = PInfo.split(',')


		# option info --------------------------------------------------
		for option in egd.readlines():

			option = option.rstrip()

			if option == "":
				continue

			# load options -----------------------------------------
			type, params = option.split('^')


			# specific types ---------------------------------------
			if type == 'Label':

				label, tip = params.split(',')
				self.build_option(optsTab('',1,type,label,'',tip,s))


			elif type == 'FileChooserButton':

				order, label, switch, tip, action = params.split(',')
				self.build_option(optsTab('',order,type,label,switch,tip,s,action))

			elif type == 'SpinButton' or type == 'HScale':

				order, label, switch, tip, x1, x2, x3 = params.split(',')
				self.build_option(optsTab('',order,type,label,switch,tip,s,x1,x2,x3))

			else:
				# common/shared types --------------------------
				order, label, switch, tip = params.split(',')
				self.build_option(optsTab('',order,type,label,switch,tip,s))

		egd.close()
		self.egdfile = filename


	# sort by display_order ================================================
	def sort_func(self, x, y):

		if x.display < y.display:
			return -1

		elif x.display == y.display:
			return 0

		else:
			return 1

	# write definition file ================================================
	def save_definition(self, filename):

		try:
			egd = open(filename, "w")

		except IOError:

			print "ERROR: Cannot save file", filename
			return

		info = 'Program^' + self.PName + ',' + self.PComm + ',' + self.PDesc

		print info
		egd.write(info)
		egd.write('\n')

		display_order = []
		display_order = self.Options.values()
		display_order.sort(self.sort_func)

#		for opt in self.Options.values():

		for opt in display_order:

			# common -----------------------------------------------
			if opt.type == 'Label':

				info = opt.type + '^' + opt.label + ',' + opt.tip

			else:
				info = opt.type + '^' + str(opt.order) + ',' + opt.label
				info += ',' + opt.switch + ',' + opt.tip

			# extras -----------------------------------------------
			if opt.type == 'FileChooserButton':
				info += ',' + opt.ex1

			if opt.type == 'SpinButton' or opt.type == 'HScale':
				info += ',' + opt.ex1 + ',' + opt.ex2 + ',' + opt.ex3

			print info
			egd.write(info)
			egd.write('\n')


		egd.close()


	# add widget and option info from optsTab object =======================
	def build_option(self, obj, rebuild=False):

		name = ""

		type = obj.type
		order = obj.order
		label = obj.label
		switch = obj.switch
		tip = obj.tip

		ex1 = obj.ex1
		ex2 = obj.ex2
		ex3 = obj.ex3

		s = self.slot

		name = type + ':' + switch + ':' + label

		if obj.type == "Label":

# TODO: while has_key(name) make a new name

			name = type + ':' + label

			self.Widgets[name] = gtk.Label("<b>" + label + "</b>")
			self.Widgets[name].set_use_markup(True)
			self.vb.pack_start(self.Widgets[name], True, True, 2)
			self.Widgets[name].show()

			self.tooltips.set_tip(self.Widgets[name], tip)

			if not rebuild:
				self.Options[name] = optsTab(name, 0, obj.type, label, '', '', s)


		elif obj.type == "CheckButton":

			if not rebuild:
				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s)

			self.Widgets[name] = gtk.CheckButton(label)
			self.vb.pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[name], tip)
			self.Widgets[name].show()

#		elif obj.type == "RadioButton":

#			if not rebuild:
#				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s, ex1)

#			self.Widgets[name] = gtk.RadioButton(ex1, label)
#			self.vb.pack_start(self.Widgets[name], True, True, 2)
#			self.tooltips.set_tip(self.Widgets[name], tip)
#			self.Widgets[name].show()

		elif obj.type == "Entry":

			hbox = name + "_hbx"
			elab = name + "_lab"

			self.Widgets[hbox] = gtk.HBox(False, 8)
			self.vb.pack_start(self.Widgets[hbox], True, True, 2)
			self.Widgets[hbox].show()

			self.Widgets[elab] = gtk.Label(label)
			self.Widgets[hbox].pack_start(self.Widgets[elab], False, False, 2)
			self.Widgets[elab].show()

			if not rebuild:
				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s)

			self.Widgets[name] = gtk.Entry()

			self.Widgets[hbox].pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[name], tip)
			self.Widgets[name].show()

		elif obj.type == "FileChooserButton":

			hbox = name + "_hbx"
			elab = name + "_lab"

			self.Widgets[hbox] = gtk.HBox(False, 8)
			self.vb.pack_start(self.Widgets[hbox], True, True, 2)
			self.Widgets[hbox].show()

			self.Widgets[elab] = gtk.Label(label)
			self.Widgets[hbox].pack_start(self.Widgets[elab], False, False, 2)
			self.Widgets[elab].show()

			if not rebuild:

				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s, ex1)

			self.Widgets[name] = gtk.FileChooserButton(tip)

#			os.environ('HOME')
			self.Widgets[name].set_filename('/')

			if ex1 == 'SELECT_FOLDER':
				self.Widgets[name].set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)

			self.Widgets[hbox].pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[name], tip)
			self.Widgets[name].show()


		elif obj.type == "SpinButton":

			hbox = name + "_hbx"
			elab = name + "_lab"

			self.Widgets[hbox] = gtk.HBox(False, 8)
			self.vb.pack_start(self.Widgets[hbox], True, True, 2)
			self.Widgets[hbox].show()

			self.Widgets[elab] = gtk.Label(label)
			self.Widgets[hbox].pack_start(self.Widgets[elab], False, False, 2)
			self.Widgets[elab].show()

			if not rebuild:

				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s, ex1, ex2, ex3)

			self.Widgets[name] = gtk.SpinButton()
			self.Widgets[name].set_update_policy(gtk.UPDATE_ALWAYS)
			self.Widgets[name].set_increments(1,1)
			self.Widgets[name].set_numeric(True)
			self.Widgets[name].set_range(int(ex2), int(ex3))
			self.Widgets[name].set_value(int(ex1))
#			self.widgets[name].set_digits(0)

			self.Widgets[hbox].pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[name], tip)
			self.Widgets[name].show()

		elif obj.type == "HScale":

			hbox = name + "_hbx"
			elab = name + "_lab"

			self.Widgets[hbox] = gtk.HBox(False, 8)
			self.vb.pack_start(self.Widgets[hbox], True, True, 2)
			self.Widgets[hbox].show()

			self.Widgets[elab] = gtk.Label(label)
			self.Widgets[hbox].pack_start(self.Widgets[elab], False, False, 2)
			self.Widgets[elab].show()

			if not rebuild:

				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s, ex1, ex2, ex3)

			self.Widgets[name] = gtk.HScale()
			self.Widgets[name].set_value_pos(gtk.POS_LEFT)
			self.Widgets[name].set_range(int(ex2), int(ex3))
			self.Widgets[name].set_value(int(ex1))
			self.Widgets[name].set_digits(0)

			self.Widgets[hbox].pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[elab], tip)
			self.Widgets[name].show()

		elif obj.type == 'Button':

			if not rebuild:

				self.Options[name] = optsTab(name, order, obj.type, label, switch, tip, s)

			self.Widgets[name] = gtk.Button(label)
			self.vb.pack_start(self.Widgets[name], True, True, 2)
			self.tooltips.set_tip(self.Widgets[name], tip)
			self.Widgets[name].show()

#		self.Widgets[name].connect("button_press_event", self.press_option, name)
		self.cbOption.append_text(name)

		self.slot += 1
		return name

	# ======================================================================
	# callback functions ===================================================

#	def press_option(self, widget, name):

#		self.Widgets[self.selected].drag_unhighlight()
#		self.Widgets[name].drag_highlight()

	# new program selected =================================================
	def on_program_changed(self, widget, data=None):

		if self.cbProgram.get_active_text() == 'Editing Current Definition':

			return


		if os.name == 'posix':

			conf = self.egd_dir + '/'
		else:
			conf = '\\egconfig\\'

		self.clear_all()
		self.open_definition(conf + self.appconf[self.cbProgram.get_active_text()])

	# add a new program ====================================================
	def addprog_clicked(self, widget, data=None):

		# just like new file, but prompt with program info -------------
		self.clear_all()
		self.cbProgram.set_active(0)
		self.proginfo_clicked(widget)


	# new option selected in option combo box ==============================
	def option_changed(self, widget, data=None):

		if self.cbOption.get_active_text() == 'Nothing selected':

			self.none_selected()

		else:
			if self.selected != '' and self.Widgets.has_key(self.selected):
				self.Widgets[self.selected].drag_unhighlight()

			self.selected = self.cbOption.get_active_text()
			self.new_selection(self.Options[self.selected])


	# delete currently selected option =====================================
	def delete_clicked(self, widget, data=None):

		del self.Options[self.selected]

		for widget in self.Widgets.values():

			widget.destroy()

		self.Widgets.clear()
		self.slot = 0

		self.none_selected()

		self.cbOption.get_model().clear()
		self.cbOption.append_text('Nothing selected')
		self.cbOption.set_active(0)

		display_order = []
		display_order = self.Options.values()
		display_order.sort(self.sort_func)

		for option in display_order:

			self.build_option(option, True)

	# start fresh ==========================================================
	def new_clicked(self, widget, data=None):

		self.clear_all()
		self.cbProgram.set_active(0)

	# prompt for file to load ==============================================
	def open_clicked(self, widget, data=None):

		chooser = gtk.FileChooserDialog("Open EveryGui Definition File", None, gtk.FILE_CHOOSER_ACTION_OPEN,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		chooser.set_default_response(gtk.RESPONSE_OK)
		chooser.set_select_multiple(True)

		chooser.set_current_folder(self.egd_dir)

		filter = gtk.FileFilter()
		filter.set_name("EveryGui Definition (*.egd)")
		filter.add_pattern("*.egd")
		chooser.add_filter(filter)

		response = chooser.run()
		if response == gtk.RESPONSE_OK:

			egdfile = chooser.get_filename()
			self.open_definition(egdfile)

		chooser.destroy()
		self.cbProgram.set_active(0)

	# save currently opened file ===========================================
	def save_clicked(self, widget, data=None):

		if self.egdfile != '':
			self.save_definition(self.egdfile)
		else:
			self.saveas_clicked(widget)

	# prompt for filename and save =========================================
	def saveas_clicked(self, widget, data=None):

		chooser = gtk.FileChooserDialog("Save EveryGui Definition File", None, gtk.FILE_CHOOSER_ACTION_SAVE,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		chooser.set_default_response(gtk.RESPONSE_OK)
		chooser.set_current_folder(self.egd_dir)

		append = '.egd'

		filter = gtk.FileFilter()
		filter.set_name("EveryGui Definition (*.egd)")
		filter.add_pattern("*.egd")
		chooser.add_filter(filter)

		response = chooser.run()
		if response == gtk.RESPONSE_OK:

			egdfile = chooser.get_filename()
			extmatch = re.search(append, egdfile, re.IGNORECASE)

			if not extmatch:
				egdfile += append

			self.save_definition(egdfile)
			self.egdfile = egdfile

		chooser.destroy()

	# apply changes ========================================================
	def editopt_clicked(self, widget, data=None):

		self.update_option()

	# edit program info ====================================================
	def proginfo_clicked(self, widget, data=None):

		InitPI = [self.PName, self.PComm, self.PDesc]

		Pi = DlgProg('/usr/local/share/everygui/ui/egdesigner.glade', "dlgProgInfo", InitPI)
		Pi.dlgProgInfo.hide()

		resp = Pi.dlgProgInfo.run()
		if resp == gtk.RESPONSE_OK:

			Info = Pi.get_info()

			self.PName = Info[0]
			self.PComm = Info[1]
			self.PDesc = Info[2]

		Pi.dlgProgInfo.destroy()

	# new options ==========================================================
	def label_clicked(self, widget, data=None):
		self.new_option("Label")

	def checkbutton_clicked(self, widget, data=None):
		self.new_option("CheckButton")

	def radiobutton_clicked(self, widget, data=None):
		self.new_option("RadioButton")

	def entry_clicked(self, widget, data=None):
		self.new_option("Entry")

	def filechooser_clicked(self, widget, data=None):
		self.new_option("FileChooserButton")

	def spinbutton_clicked(self, widget, data=None):
		self.new_option("SpinButton")

	def hscale_clicked(self, widget, data=None):
		self.new_option("HScale")

	def execbutton_clicked(self, widget, data=None):
		self.new_option("Button")

	# exit program =========================================================
	def on_appWnd_delete_event(self, widget, event, data=None):
		gtk.main_quit()

	def on_destroy(self, widget, data=None):
		gtk.main_quit()

	# end callback funtions ================================================
	# ======================================================================

	# about everygui =======================================================
	def goto_url(self, dialog, website, data=None):

		if os.name == 'posix':
			browser = 'mozilla '
		else:
			browser = 'explorer '

		visit = browser + website + ' &'
		os.system(visit)

	def about_clicked(self, widget, data=None):

		try:
			file = open("/usr/local/share/everygui/docs/COPYING", "r")
			license = file.read()
		except IOError:
			print 'Cannot open license file for reading'
			license = "GNU General Public License (GPL)"

		authors = ["EveryGUI by Jeffrey Bakker", "GladeWrapper by Padraig Brady"]

		about = gtk.AboutDialog()

		gtk.about_dialog_set_url_hook(self.goto_url, "http://everygui.sf.net")

		about.set_name("EveryGUI Designer")
		about.set_version("0.99.b")
		about.set_copyright("(C)2006 Jeffrey Bakker")
		about.set_authors(authors)
		about.set_license(license)
		about.set_website("http://everygui.sf.net")
		about.set_website_label("http://everygui.sf.net")

		about.run()
		about.destroy()


# start the application ========================================================
DesignWnd = Designer("/usr/local/share/everygui/ui/egdesigner.glade", "designerWnd")

gtk.main()
# end of program ===============================================================
