#!/usr/bin/env python

# EveryGui (C)2006 Jeffrey Bakker
#
# EveryGui is a GUI for batch conversion with command line programs


# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import libglade
import pango
import gtk

import shutil
import os
import re

import threading
import locale
locale.setlocale(locale.LC_ALL,'')


# option info ==================================================================
class optsTab:

	def __init__(self, name, order, type, label, switch, tip):

		self.name = name
		self.order = int(order)
		self.type = type
		self.label = label
		self.switch = switch
		self.tip = tip


# options class ================================================================
class optsGui:

	def __init__(self, box, config, buff):

		self.box = box
		self.tooltips = gtk.Tooltips()
		self.buff = buff

		self.widgets = {}
		self.options = {}

		self.load_config(config)

	# destroy widgets ======================================================
	def destroy_widgets(self):

		self.tooltips.destroy()
		del self.tooltips

		for widget in self.widgets.values():

			widget.destroy()

		self.widgets.clear()
		self.options.clear()

		del self.box

	# display program output ===============================================
	def read_output(self, buffer, command):

		buffer.set_text('Executing `' + command + '`:\n\n')
		stdin, stdouterr = os.popen4(command)

		for line in stdouterr.readlines():
			buffer.insert(buffer.get_end_iter(), line)

	# quick execute ========================================================
	def quick_exec(self, widget, command):

		self.read_output(self.buff, command)
#		notebook1.set_current_page(2)
#		os.system(command)

	# read egd configuration file to create gui ============================
	def load_config(self, file):

		try:

			# everygui definition file
			egdef = open(file, "r")

		except IOError:

			print "Cannot open config file", file
			return

		config = egdef.readlines()

		# todo use dictionary to bind switch to widget for get_options

		for option in config:

			option = option.rstrip()

			if option == "":

				continue

			# split element from parameters
			element, params = option.split('^')

			if element == "Label":

				# split parameters
				label, tip = params.split(',')

				name = "lab_" + label
#				rule = "rul_" + label

				self.widgets[name] = gtk.Label("<b>" + label + "</b>")
				self.widgets[name].set_use_markup(True)
				self.box.pack_start(self.widgets[name], True, True, 2)
				self.widgets[name].show()

#				self.widgets[rule] = gtk.HRuler()
#				self.box.pack_start(self.widgets[rule], True, True, 2)
#				self.widgets[rule].show()

				self.tooltips.set_tip(self.widgets[name], tip)


			if element == "CheckButton":

				# split parameters
				order, label, switch, tip = params.split(',')

				name = "chk_" + switch + " " + label

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.CheckButton(label)
				self.box.pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[name], tip)
				self.widgets[name].show()

			elif element == "Entry":

				# split parameters
				order, label, switch, tip = params.split(',')

				name = "ent_" + switch + " " + label
				hbox = name + "_hbx"
				elab = name + "_lab"

				self.widgets[hbox] = gtk.HBox(False, 8)
				self.box.pack_start(self.widgets[hbox], True, True, 2)
				self.widgets[hbox].show()

				self.widgets[elab] = gtk.Label(label)
				self.widgets[hbox].pack_start(self.widgets[elab], False, False, 2)
				self.widgets[elab].show()

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.Entry()

				self.widgets[hbox].pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[name], tip)
				self.widgets[name].show()

			elif element == "FileChooserButton":

				# split parameters
				order, label, switch, tip, action = params.split(',')

				name = "fc_" + switch + " " + label
				hbox = name + "_hbx"
				elab = name + "_lab"

				self.widgets[hbox] = gtk.HBox(False, 8)
				self.box.pack_start(self.widgets[hbox], True, True, 2)
				self.widgets[hbox].show()

				self.widgets[elab] = gtk.Label(label)
				self.widgets[hbox].pack_start(self.widgets[elab], False, False, 2)
				self.widgets[elab].show()

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.FileChooserButton(tip)

#				os.environ('HOME')
				self.widgets[name].set_filename('/')

				if action == 'SELECT_FOLDER':
					self.widgets[name].set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)


				self.widgets[hbox].pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[name], tip)
				self.widgets[name].show()

			elif element == "SpinButton":

				# split parameters
				order, label, switch, tip, val, min, max = params.split(',')

				name = "spn_" + switch + " " + label
				hbox = name + "_hbx"
				elab = name + "_lab"

				self.widgets[hbox] = gtk.HBox(False, 8)
				self.box.pack_start(self.widgets[hbox], True, True, 2)
				self.widgets[hbox].show()

				self.widgets[elab] = gtk.Label(label)
				self.widgets[hbox].pack_start(self.widgets[elab], False, False, 2)
				self.widgets[elab].show()

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.SpinButton()
				self.widgets[name].set_update_policy(gtk.UPDATE_ALWAYS)
				self.widgets[name].set_increments(1,1)
				self.widgets[name].set_numeric(True)
				self.widgets[name].set_range(int(min), int(max))
				self.widgets[name].set_value(int(val))
#				self.widgets[name].set_digits(0)

				self.widgets[hbox].pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[name], tip)
				self.widgets[name].show()

			elif element == "HScale":

				# split parameters
				order, label, switch, tip, val, min, max = params.split(',')

				name = "scl_" + switch + " " + label
				hbox = name + "_hbx"
				elab = name + "_lab"

				self.widgets[hbox] = gtk.HBox(False, 8)
				self.box.pack_start(self.widgets[hbox], True, True, 2)
				self.widgets[hbox].show()

				self.widgets[elab] = gtk.Label(label)
				self.widgets[hbox].pack_start(self.widgets[elab], False, False, 2)
				self.widgets[elab].show()

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.HScale()
				self.widgets[name].set_value_pos(gtk.POS_LEFT)
				self.widgets[name].set_range(int(min), int(max))
				self.widgets[name].set_value(int(val))
				self.widgets[name].set_digits(0)

				self.widgets[hbox].pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[elab], tip)
				self.widgets[name].show()

			elif element == "Button":

				# split parameters
				order, label, switch, tip = params.split(',')

				name = "btn_" + switch + " " + label

				self.options[name] = optsTab(name, order, element, label, switch, tip)
				self.widgets[name] = gtk.Button(label)
				self.box.pack_start(self.widgets[name], True, True, 2)
				self.tooltips.set_tip(self.widgets[name], tip)
				self.widgets[name].show()

				self.widgets[name].connect("clicked", self.quick_exec, switch)

		egdef.close()
		return

	# formulate string of command line options ============================
	def get_options(self):

		stropts = ''

		for order in range(0,4):

			for opts in self.options.values():

				if opts.type == "CheckButton":

					if opts.order == order and self.widgets[opts.name].get_active():
						stropts += ' ' + opts.switch

				elif opts.type == "Entry":

					user_entry = self.widgets[opts.name].get_text()
					if opts.order == order and user_entry != "":
						stropts += ' ' + opts.switch + '"' + user_entry + '"'

				elif opts.type == "FileChooserButton":

					user_file = self.widgets[opts.name].get_filename()
					if opts.order == order and user_file != None:
						stropts += ' ' + opts.switch + '"' + user_file + '"'

				elif opts.type == "SpinButton":

					if opts.order == order:
						stropts += ' ' + opts.switch + str(self.widgets[opts.name].get_value_as_int())

				elif opts.type == "HScale":

					if opts.order == order:
						stropts += ' ' + opts.switch + str(self.widgets[opts.name].get_value())

		return stropts

# WildCard Dialog class ========================================================
class wildCard(libglade.GladeWrapper):

	def __init__(self, Filename, WindowName, start_dir="."):
		libglade.GladeWrapper.__init__(self, Filename, WindowName)

	def wildcancel_clicked(self, widget, data=None):
		self.wildDlg.hide_all()

	def wildadd_clicked(self, widget, data=None):
		self.wildDlg.hide_all()

	def get_info(self):
		return self.fcWild.get_filename() + '/' + self.entWild.get_text()


# EveryGUI class from GladeWrapper =============================================
class guiwnd(libglade.GladeWrapper):

	# grab widgets from xml ================================================
	def __init__(self, Filename, WindowName, start_dir="."):
		libglade.GladeWrapper.__init__(self, Filename, WindowName)


		self.optPage = None
		self.pgOptions = None

		self.applocs = {}
		self.appconf = {}

		self.cwd = os.getcwd()
		self.egd_dir = os.environ["HOME"] + "/.egconfig"

		if not os.path.exists(self.egd_dir):
			shutil.copytree("/usr/local/share/everygui/egconfig", self.egd_dir)

		# create notebook tabs guis ====================================
		self.xmlf = gtk.glade.XML('/usr/local/share/everygui/ui/everygui.glade', root="vbox2")
		self.pgFiles = self.xmlf.get_widget('vbox2')

		self.sw = gtk.ScrolledWindow()
		self.sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		self.sw.show()

		self.swout = gtk.ScrolledWindow()
		self.swout.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
		self.swout.show()

		self.textview = gtk.TextView()
		self.textview.set_editable(False)
		self.textview.set_cursor_visible(False)
		self.textview.set_wrap_mode(gtk.WRAP_WORD)
		self.textview.set_left_margin(4)
		self.textview.set_right_margin(4)
		self.textview.modify_font(pango.FontDescription('monospace'))
		self.textbuff = self.textview.get_buffer()
		self.swout.add(self.textview)
		self.textview.show()

		label1 = gtk.Label("Input Files")
		self.notebook1.append_page(self.pgFiles, label1)
		label1.show()

		label2 = gtk.Label("Options")
		self.notebook1.append_page(self.sw, label2)
		label2.show()

		label3 = gtk.Label("Output")
		self.notebook1.append_page(self.swout, label3)
		label3.show()

		self.notebook1.set_current_page(0)


		# initialize treeview ==========================================
		self.FileList = self.xmlf.get_widget('tvFiles')

		self.files_model = gtk.ListStore(str)
		self.FileList.set_model(self.files_model)
		files_column = gtk.TreeViewColumn("Files to process", gtk.CellRendererText(), text=0)
		self.FileList.append_column(files_column)

		self.FileList.enable_model_drag_dest([('text/plain', 0, 0)], gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_COPY)
		self.FileList.connect("drag-data-received", self.drag_data_received_cb)

#		self.mini_model = gtk.ListStore(str)
#		self.cbMini.set_model(self.mini_model)

		# read program configs into dictionaries and combo box =========
		self.cbProgram.remove_text(0)

		os.chdir(self.egd_dir)
		egd_dir = os.listdir('.')
		egd_dir.sort()

		for egd in egd_dir:

			try:
				egd_file = open(egd, "r")

			except IOError:
				print "Cannot open file", egd

			app_info = egd_file.readline()
#			print "App info:", app_info

			dump, info = app_info.split('^')
			appname, location, about = info.split(',')

			self.appconf[appname] = egd
			self.applocs[appname] = location
			self.cbProgram.append_text(appname)

			egd_file.close()

		os.chdir(self.cwd)

		self.cbProgram.set_active(0)

		# connect toolbar signals ======================================
		self.tbQuit.connect("clicked", self.on_destroy)
		self.tbRun.connect("clicked", self.run_clicked)
		self.tbAdd.connect("clicked", self.add_clicked)
		self.tbDir.connect("clicked", self.dir_clicked)
		self.tbRemove.connect("clicked", self.remove_clicked)
		self.tbAbout.connect("clicked", self.about_clicked)

		# connect menubar signals ======================================
		self.add1.connect("activate", self.add_clicked)
		self.dir1.connect("activate", self.dir_clicked)
		self.add_wildcard1.connect("activate", self.wild_clicked)
		self.rem1.connect("activate", self.remove_clicked)
		self.quit1.connect("activate", self.on_destroy)

		self.run1.connect("activate", self.run_clicked)
		self.script1.connect("activate", self.script_clicked)
		self.config1.connect("activate", self.config_clicked)
		self.clear1.connect("activate", self.clear_output)

		self.about1.connect("activate", self.about_clicked)

		# connect button signals =======================================
		self.xmlf.get_widget('btnAdd').connect("clicked", self.add_clicked)
		self.xmlf.get_widget('btnDir').connect("clicked", self.dir_clicked)
		self.xmlf.get_widget('btnRemove').connect("clicked", self.remove_clicked)
		self.xmlf.get_widget('btnEmpty').connect("clicked", self.empty_clicked)
		self.xmlf.get_widget('btnWild').connect("clicked", self.wild_clicked)

		self.btnProgs.connect("clicked", self.config_clicked)
		self.cbMini.child.connect("key-press-event", self.mini_keypress)

	def drag_data_received_cb(self, treeview, context, x, y, selection, info, timestamp):

		drop_info = treeview.get_dest_row_at_pos(x, y)

		print "dropped"

		if drop_info:

			path, position = drop_info
			data = selection.data

			iter = self.files_model.append()
			self.files_model.set(iter,0,data)


	# new program selected =================================================
	def on_program_changed(self, widget, data=None):

		# destroy all previous widgets =================================
		if self.optPage != None:

			self.optPage.destroy_widgets()
			del self.optPage

			self.pgOptions.destroy()
			del self.pgOptions

			self.pgOptions = None
			self.optPage = None

		# new widget container ===========
		self.pgOptions = gtk.VBox(True, 2)
		self.pgOptions.set_border_width(8)
		self.pgOptions.show()

		os.chdir(self.egd_dir)

		# dynamically load widgets ==============
		self.sw.add_with_viewport(self.pgOptions)
		self.optPage = optsGui(self.pgOptions, self.appconf[self.cbProgram.get_active_text()], self.textbuff)

		os.chdir(self.cwd)

	# clear output window ==================================================
	def clear_output(self, widget):
		self.textbuff.set_text("")

	# display program output ===============================================
	def read_output(self, buffer, command):

		buffer.set_text('Executing `' + command + '`:\n\n')

		encoding = locale.getpreferredencoding()
		utf8conv = lambda x : unicode(x, encoding).encode('utf8')

		stdin, stdouterr = os.popen4(command)

		for line in stdouterr.readlines():
			buffer.insert(buffer.get_end_iter(), utf8conv(line))
#			self.textview.scroll_to_iter(buffer.get_end_iter(), 0.2)

	# run the command ======================================================
	def batch_command(self, model, path, iter, script):

		formula = self.applocs[self.cbProgram.get_active_text()]

		command = ''
		words = formula.split(' ')

		for word in words:

			if word == "$options":
				command += self.optPage.get_options() + ' '

			elif word == "$infile":
				command += '"' + model.get_value(iter, 0) + '" '

			else:
				command += word + " "

#		self.textbuff.insert(self.textbuff.get_end_iter(), command)
		print command

		self.cbMini.append_text(command)

		# execute command
		if(script == None):
			thr = threading.Thread(target= self.read_output, args=(self.textbuff, command))
			thr.run()

		# write command to script
		else:
			script.write(command)
			script.write('\n')

	# get files as list ====================================================
	def files_as_list(self, model, path, iter, flist):

		flist.append(model.get_value(iter, 0))

	# run command from mini console ========================================
	def minigo_clicked(self, widget, data=None):

		if self.cbMini.child.get_text() == 'clear':
			self.textbuff.set_text("")

		elif self.cbMini.child.get_text() == 'Delete History':
			self.cbMini.get_model().clear()

		else:
			self.read_output(self.textbuff, self.cbMini.child.get_text())
			self.notebook1.set_current_page(2)

		self.cbMini.append_text(self.cbMini.child.get_text())
		self.cbMini.child.set_text('')

	def mini_keypress(self, widget, event, data=None):

		if 'Return' == gtk.gdk.keyval_name(event.keyval):
#		if event.keyval == gtk.gdk.keyval_from_name("Return"):

			# Enter pressed
			self.minigo_clicked(self.btnMiniGo)

	# run the command ======================================================
	def run_command(self, script):

		formula = self.applocs[self.cbProgram.get_active_text()]

		command = ''
		words = formula.split(' ')

		for word in words:

			if word == "$options":
				command += self.optPage.get_options() + ' '

			elif word == "$infile":

				if not self.xmlf.get_widget('chkList').get_active():
					continue


				flist = []
				self.files_model.foreach(self.files_as_list, flist)

				for infile in flist:
					command += infile + ' '
			else:
				command += word + ' '

		self.cbMini.append_text(command)

		# execute command
		if(script == None):

			thr = threading.Thread(target= self.read_output, args=(self.textbuff, command))
			thr.run()

		# write command to script
		else:
			script.write(command)
			script.write('\n')

	# run was clicked ======================================================
	def run_clicked(self, widget, data=None):

		print "Run was clicked."

		if self.files_model.get_iter_root() != None:

			# use input files
			if self.xmlf.get_widget('chkList').get_active():

				# pass files as list
				self.run_command(None)
			else:
				# run command for each file
				self.files_model.foreach(self.batch_command, None)

		else:
			# no input files
			self.run_command(None)

		return

	# script was clicked ===================================================
	def script_clicked(self, widget, data=None):

		print "Script was clicked."

		chooser = gtk.FileChooserDialog("Save Script", None, gtk.FILE_CHOOSER_ACTION_SAVE,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_SAVE, gtk.RESPONSE_OK))

		chooser.set_default_response(gtk.RESPONSE_OK)

		# OS Specific
		if os.name == 'posix':

			filter = gtk.FileFilter()
			filter.set_name("Shell script (*.sh)")
			filter.add_pattern("*.sh")
			chooser.add_filter(filter)

			append = ".sh"

		else:

			filter = gtk.FileFilter()
			filter.set_name("Batch file (*.bat)")
			filter.add_pattern("*.bat")
			chooser.add_filter(filter)

			append = ".bat"


		filter = gtk.FileFilter()
		filter.set_name("All files (*)")
		filter.add_pattern("*")
		chooser.add_filter(filter)

		response = chooser.run()
		if response == gtk.RESPONSE_OK:

			script = chooser.get_filename()
			extmatch = re.search(append, script, re.IGNORECASE)

			if not extmatch:
				script += append

			self.open_script(script)

		chooser.destroy()

		return

	# open script for writing ==============================================
	def open_script(self, script):

		try:
			file = open(script, "w")

		except IOError:

			print 'Cannot save script file.'
			return

		# OS Specific
		if os.name == 'posix':
			file.write("#!/bin/sh\n\n")


		# write script lines =======================
		if self.files_model.get_iter_root() != None:

			# use input files
			self.files_model.foreach(self.batch_command, file)

		else:
			# no input files
			self.run_command(file)


		# OS Specific
		if os.name != 'posix':
			file.write("PAUSE\n")

		print script, "script saved."
		file.close()


	# remove file from list ================================================
	def remove_clicked(self, widget, data=None):

		sel = self.FileList.get_selection()
		model, iter = sel.get_selected()
		iter = model.remove(iter)
		return


	# clear input files list ===============================================
	def empty_clicked(self, widget, data=None):
		self.files_model.clear()


	# add input file to list ===============================================
	def add_file(self, filename):
		iter = self.files_model.append()
		self.files_model.set(iter,0,filename)


	# create a filechooser for input files =================================
	def add_clicked(self, widget, data=None):
		chooser = gtk.FileChooserDialog("Input File", None, gtk.FILE_CHOOSER_ACTION_OPEN,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		chooser.set_default_response(gtk.RESPONSE_OK)
		chooser.set_select_multiple(True)

		filter = gtk.FileFilter()
		filter.set_name("All files")
		filter.add_pattern("*")
		chooser.add_filter(filter)

		response = chooser.run()
		if response == gtk.RESPONSE_OK:

			for files in chooser.get_filenames():
				self.add_file(files)

		chooser.destroy()


	# add directory of input files to list =================================
	def add_dir(self, directory):

		contents = os.listdir(directory)

		# OS Specific
		if os.name == 'posix':
			folder = "/"
		else:
			folder = "\\"

		for files in contents:

			iter = self.files_model.append()
			path = directory + folder + files
			self.files_model.set(iter,0,path)

	# create a filechooser for input files =================================
	def dir_clicked(self, widget, data=None):
		chooser = gtk.FileChooserDialog("Locate Directory",
						None, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		chooser.set_default_response(gtk.RESPONSE_OK)

		response = chooser.run()
		if response == gtk.RESPONSE_OK:
			self.add_dir(chooser.get_filename())

		chooser.destroy()

	# add a wildcard to files list =========================================
	def wild_clicked(self, widget, data=None):

		add = wildCard('/usr/local/share/everygui/ui/everygui.glade', "wildDlg")
		add.wildDlg.hide()

		resp = add.wildDlg.run()
		if resp == gtk.RESPONSE_OK:

			iter = self.files_model.append()
			self.files_model.set(iter,0,add.get_info())

		add.wildDlg.destroy()

	# launch EveryGUI Designer =============================================
	def config_clicked(self, widget, data=None):

		os.system("python /usr/local/src/everygui/designer.py &")

	# exit program =========================================================
	def on_appWnd_delete_event(self, widget, event, data=None):
		gtk.main_quit()

	def on_destroy(self, widget, data=None):
		gtk.main_quit()


	# about everygui =======================================================
	def goto_url(self, dialog, website, data=None):

		if os.name == 'posix':
			browser = 'mozilla '
		else:
			browser = 'explorer '

		visit = browser + website + ' &'
		os.system(visit)

	def about_clicked(self, widget, data=None):

		try:
			file = open("/usr/local/share/everygui/docs/COPYING", "r")
			license = file.read()
		except IOError:
			print 'Cannot open license file for reading'
			license = "GNU General Public License (GPL)"

		authors = ["EveryGUI by Jeffrey Bakker", "GladeWrapper by Padraig Brady"]

		about = gtk.AboutDialog()

		gtk.about_dialog_set_url_hook(self.goto_url, "http://everygui.sf.net")

		about.set_name("EveryGUI Chameleon")
		about.set_version("0.99.b")
		about.set_copyright("(C)2006 Jeffrey Bakker")
		about.set_authors(authors)
		about.set_license(license)
		about.set_website("http://everygui.sf.net")
		about.set_website_label("http://everygui.sf.net")

		about.run()
		about.destroy()


# start the application ========================================================
EveryGui = guiwnd("/usr/local/share/everygui/ui/everygui.glade", "appWnd")

gtk.main()
# end of program ===============================================================
